@@ -48,110 +48,6 @@
 }
 
 
-wpgma_weights <- function(tree){
-  ntips <- Ntip(tree)
-  tree_u <- unroot(tree)
-  ind <- sort(unique( tree_u$edge[,1]) )[-1]
-  desc <- Descendants(tree_u, ind)
-  x <- seq_len(ntips)
-  w <- integer( ntips * (ntips-1L) / 2 )
-  for(i in seq_along(ind)){
-    ind <- getIndex(desc[[i]], x[-desc[[i]]], ntips)
-    w[ind] <- w[ind] + 1L
-  }
-  w
-}
-
-upgma.edge.length <- function(x, dm, method = "average") {
-  METHODS <- c("average", "single", "complete", "mcquitty")
-  i.meth <- match.arg(method, METHODS)
-  X <- designTree(x, "rooted", TRUE)
-  labels <- x$tip
-  if (is.matrix(dm) || inherits(dm, "dist")) {
-    dm <- as.matrix(dm)[labels, labels]
-    dm <- dm[lower.tri(dm)]
-  }
-  ind <- X@i + 1
-  node <- X@p
-  X@x[] <- 1
-  nh <- numeric(max(x$edge))
-  Y <- X * dm
-  if(i.meth=="mcquitty"){
-    w <- .5 ^ wpgma_weights(x)
-    Y <- X * (dm * w)
-    Y[,1] <- Y[,1]/2
-#    Y = as.matrix(X)
-#    beta = solve( t(Y) %*% diag(w) %*% Y ) %*% (t(Y) %*% diag(w) %*% y)
-#    nh[X@nodes] <- beta
-#    x$edge.length <- (nh[x$edge[, 1]] - nh[x$edge[, 2]]) / 2
-#    return( x )
-  }
-
-  for (i in 1:(length(node) - 1)) {
-    pos <- ind[(node[i] + 1):node[i + 1]]
-    tmp <- switch(i.meth,
-      average = mean(Y[pos, i]),
-      mcquitty = sum(Y[pos, i]),
-      single = min(Y[pos, i]),
-      complete = max(Y[pos, i]))
-    nh[X@nodes[i]] <- tmp
-  }
-  x$edge.length <- (nh[x$edge[, 1]] - nh[x$edge[, 2]]) / 2
-  x
-}
-
-
-upgma_nni <- function(d, method = "average", opt = "min", trace = 0,
-                      mc.cores = 2L){
-  METHODS <- c("average", "single", "complete", "mcquitty")
-  method <- match.arg(method, METHODS)
-  OPT <- c("min", "ls")
-  opt <- match.arg(opt, OPT)
-  tree <- upgma(d, method = method)
-  labels <- tree$tip.label
-  nTips <- length(labels)
-  y <- as.matrix(d)[labels, labels]
-  y <- y[lower.tri(y)]
-  best.tree <- tree
-  bestLS <- sum( (coph(best.tree) - y)^2)
-  bestME <- sum(best.tree$edge.length)
-  run.nni <- TRUE
-  count_nni <- 0
-  if (trace > 0) print(count_nni)
-  while (run.nni) {
-    trees <- nni(best.tree)
-    trees <- .uncompressTipLabel(trees)
-    trees <- unclass(trees)
-    nni.trees <- lapply(trees, upgma.edge.length, y, method = method)
-    ind <- which(vapply(nni.trees, function(x) !any(x$edge.length < 0), FALSE))
-    if (length(ind) == 0) return(best.tree)
-    nni.trees <- nni.trees[ind]
-    if (opt == "min") {
-      ME <- vapply(nni.trees, function(x) sum(x$edge.length), 0)
-      if (any(ME < bestME)) {
-        bestME <- min(ME)
-        count_nni <- count_nni + 1
-        best.tree <- nni.trees[[which.min(ME)]]
-        if (trace > 0) print(bestME)
-      }
-      else run.nni <- FALSE
-    }
-    else {
-      LS <- vapply(nni.trees, function(x) sum( (coph(x) - y)^2), 0)
-      if (any(LS < bestLS)) {
-        bestLS <- min(LS)
-        count_nni <- count_nni + 1
-        best.tree <- nni.trees[[which.min(LS)]]
-        if (trace > 0) print(bestLS)
-      }
-      else run.nni <- FALSE
-    }
-  }
-  best.tree
-}
-
-
-
 #' Neighbor-Joining
 #'
 #' This function performs the neighbor-joining tree estimation of Saitou and
@@ -207,12 +103,13 @@ UNJ <- function(x){
   w <- rep(1, l)
   while (l > 2) {
     r <- rowSums(d) / (l - 2)
-    i <- 0
-    j <- 0
-    tmp <- .C("out", as.double(d), as.double(r), as.integer(l), as.integer(i),
-              as.integer(j))
-    e2 <- tmp[[5]]
-    e1 <- tmp[[4]]
+#    i <- 0
+#    j <- 0
+    tmp <- out_cpp(d, r, l)
+#    tmp <- .C("out", as.double(d), as.double(r), as.integer(l), as.integer(i),
+#              as.integer(j))
+    e2 <- tmp[2]
+    e1 <- tmp[1]
     l1 <- d[e1, e2] / 2 + sum( (d[e1, -c(e1, e2)] - d[e2, -c(e1, e2)]) *
                                 w[-c(e1, e2)]) / (2 * (n - w[e1] - w[e2]))
     l2 <- d[e1, e2] / 2 + sum( (d[e2, -c(e1, e2)] - d[e1, -c(e1, e2)]) *
@@ -240,77 +137,6 @@ UNJ <- function(x){
 }
 
 
-PNJ <- function(data)
-{
-  q <- l <- r <- length(data)
-  weight <- attr(data, "weight")
-
-  height <- NULL
-  parentNodes <- NULL
-  childNodes <- NULL
-  nam <- names(data)
-  tip.label <- nam
-  edge <- 1:q
-
-  z <- 0
-  D <- matrix(0, q, q)
-
-  for (i in 1:(l - 1)) {
-    for (j in (i + 1):l) {
-      w <- (data[[i]] * data[[j]]) %*% c(1, 1, 1, 1)
-      D[i, j] <- sum(weight[w == 0])
-    }
-  }
-
-  while (l > 1) {
-    l <- l - 1
-    z <- z + 1
-    d <- D + t(D)
-    if (l > 1) r <- rowSums(d) / (l - 1)
-    if (l == 1) r <- rowSums(d)
-    M <- d - outer(r, r, "+")
-    diag(M) <- Inf
-
-    e <- which.min(M)
-    e0 <- e %% length(r)
-    e1 <- ifelse(e0 == 0, length(r), e0)
-    e2 <- ifelse(e0 == 0, e %/% length(r), e %/% length(r) + 1)
-
-    ind <- c(e1, e2)
-    len <- d[e] / 2
-    nam <- c(nam[-ind], as.character(-l))
-
-    parentNodes <- c(parentNodes, -l, -l)
-    childNodes <- c(childNodes, edge[e1], edge[e2])
-
-    height <- c(height, len, len)
-    edge <- c(edge[-ind], -l)
-    w <- (data[[e1]] * data[[e2]]) %*% c(1, 1, 1, 1)
-    w <- which(w == 0)
-    newDat <- data[[e1]] * data[[e2]]
-    newDat[w, ] <- data[[e1]][w, ] + data[[e2]][w, ]
-    data <- data[-c(e1, e2)]
-    data[[l]] <- newDat
-    if (l > 1) {
-      D <- as.matrix(D[, -ind])
-      D <- D[-ind, ]
-      dv <- numeric(l - 1)
-      for (i in 1:(l - 1)) {
-        w <- (data[[i]] * data[[l]]) %*% c(1, 1, 1, 1)
-        dv[i] <- sum(weight[w == 0])
-      }
-      D <- cbind(D, dv)
-      D <- rbind(D, 0)
-    }
-  }
-  tree <- list(edge = cbind(as.character(parentNodes),
-                            as.character(childNodes)), tip.label = tip.label)
-  class(tree) <- "phylo"
-  tree <- old2new.phylo(tree)
-  reorder(tree)
-}
-
-
 #
 # Distance Matrix methods
 #
@@ -332,7 +158,7 @@ PNJ <- function(data)
 #' @param splits one of "all", "star".
 #' @param dm a distance matrix.
 #' @param rooted compute a "rooted" or "unrooted" tree.
-#' @param trace defines how much information is printed during optimisation.
+#' @param trace defines how much information is printed during optimization.
 #' @param \dots further arguments, passed to other methods.
 #' @param weight vector of weights to be used in the fitting process.
 #' Weighted least squares is used with weights w, i.e., sum(w * e^2) is
@@ -377,11 +203,11 @@ designTree <- function(tree, method = "unrooted", sparse = FALSE, ...) {
 }
 
 
-# splits now work
 designUnrooted <- function(tree, order = NULL) {
   if (inherits(tree, "phylo")) {
-    if (is.rooted(tree))
-      tree <- unroot(tree)
+    if (is.rooted(tree)) tree <- unroot(tree)
+    tree <- reorder(tree, "postorder")
+#    p <- as.matrix(as.splits(tree)[tree$edge[,2]])
     p <- bipartition(tree)
   }
   if (inherits(tree, "splits")) p <- as.matrix(tree)
@@ -631,7 +457,7 @@ nnls.splits <- function(x, dm, trace = 0) {
   dm <- dm[labels, labels]
   y <- dm[lower.tri(dm)]
 
-  x <- SHORTwise(x, k)
+  x <- SHORTwise(x) #, k) # use ape version
   l <- lengths(x)
   if (any(l == 0)) x <- x[-which(l == 0)]
 
@@ -730,7 +556,7 @@ designAll <- function(n, add.split = FALSE) {
 designStar <- function(n, sparse = TRUE) {
   #    res=NULL
   #    for(i in 1:(n-1)) res = rbind(res,cbind(matrix(0,(n-i),i-1),1,diag(n-i)))
-  res <- stree(n) %>% as.splits %>% splits2design
+  res <- stree(n) |> as.splits() |> splits2design()
   if (!sparse) return(as.matrix(res))
   res
 }
