@@ -74,7 +74,7 @@ ldfactorial <- function(x) {
 #' (binary, RY-coded) or 4-state (DNA/RNA) data. \code{write.nexus.splits}
 #' writes splits returned from \code{h2st} or
 #' \code{\link[phangorn]{distanceHadamard}} to a nexus file, which can be
-#' processed by Spectronet or Splitstree.
+#' processed by Spectronet or SplitsTree.
 #'
 #' @param x a vector of length \eqn{2^n}, where n is an integer.
 #' @param v a vector of length \eqn{2^n}, where n is an integer.
@@ -116,16 +116,17 @@ ldfactorial <- function(x) {
 #' lento(fit2)
 #'
 #' # write.nexus.splits(fit2, file = "test.nxs")
-#' # read this file into Spectronet or Splitstree to show the network
+#' # read this file into Spectronet or SplitsTree to show the network
 #' \dontrun{
 #' dat <- as.character(yeast)
 #' dat4 <- phyDat(dat, type="USER", levels=c("a","c", "g", "t"), ambiguity=NULL)
 #' fit4 <- h4st(dat4)
-#'
+#' old.par <- par(no.readonly = TRUE)
 #' par(mfrow=c(3,1))
 #' lento(fit4[[1]], main="Transversion")
 #' lento(fit4[[2]], main="Transition 1")
 #' lento(fit4[[3]], main="Transition 2")
+#' par(old.par)
 #' }
 #'
 #' @rdname hadamard
@@ -145,7 +146,7 @@ hadamard <- function(x) {
 fhm <- function(v) {
   n <- length(v)
   n <- log2(n)
-  res <- .C("C_fhm", v = as.double(v), n = as.integer(n))$v #
+  res <- .Call("_phangorn_fhm_new", v = as.double(v), n = as.integer(n))
   res
 }
 
@@ -203,8 +204,7 @@ distanceHadamard <- function(dm, eps = 0.001) {
   ns <- 2^(n - 1)
   if (n > 23)
     stop("Hadamard conjugation works only efficient for n < 24")
-  result <- .Call("dist2spectra", dm, as.integer(n), as.integer(ns),
-    PACKAGE = "phangorn")
+  result <- .Call('dist2spectra', dm, as.integer(n), as.integer(ns))
   weights <- -fhm(result) / 2^(n - 2)
 
   if (eps > 0) {
