@@ -36,7 +36,8 @@ aic.weights <- function(aic) {
 #' data and the calls to allow get the estimated models, e.g. as a starting
 #' point for further analysis (see example).
 #' @author Klaus Schliep \email{klaus.schliep@@gmail.com}
-#' @seealso \code{\link{pml}}, \code{\link{anova}}, \code{\link[stats]{AIC}}
+#' @seealso \code{\link{pml}}, \code{\link{anova}}, \code{\link[stats]{AIC}},
+#' \code{\link{codonTest}}
 #' @references Burnham, K. P. and Anderson, D. R (2002) \emph{Model selection
 #' and multimodel inference: a practical information-theoretic approach}. 2nd
 #' ed. Springer, New York
@@ -85,12 +86,15 @@ modelTest <- function(object, tree = NULL, model = c("JC", "F81", "K80", "HKY",
     if (is.null(tree))
       tree <- object$tree
   }
-
+  if(!identical(sort(names(data)), sort(tree$tip.label))){
+    stop("Labels in tree and data differ!")
+  }
   if (attr(data, "type") == "DNA") type <- c("JC", "F81", "K80", "HKY", "TrNe",
       "TrN", "TPM1", "K81", "TPM1u", "TPM2", "TPM2u", "TPM3", "TPM3u",
       "TIM1e", "TIM1", "TIM2e", "TIM2", "TIM3e", "TIM3", "TVMe", "TVM",
       "SYM", "GTR")
   if (attr(data, "type") == "AA") type <- .aamodels
+  if (attr(data, "type") == "USER") type <- "JC"
 
   if ( (length(model) == 1) && model == "all") model <- type
   model <- match.arg(model, type, TRUE)
@@ -98,11 +102,11 @@ modelTest <- function(object, tree = NULL, model = c("JC", "F81", "K80", "HKY",
   env <- new.env()
   assign("data", data, envir = env)
 
-  if (is.null(tree))
-    tree <- NJ(dist.hamming(data))
-  else {
-    if (length(tree$tip.label) > 3) tree <- nnls.phylo(tree, dist.ml(data))
-    # may need something faster for trees > 500 taxa
+  if (is.null(tree)) tree <- candidate.tree(data)
+  if (is.null(tree$tip.label)){
+      tree <- acctran(tree, data)
+      tree$edge.length <- tree$edge.length / sum(attr(data, "weight"))
+      tree <- minEdge(tree, tau=1e-8)
   }
   trace <- control$trace
   control$trace <- trace - 1
