@@ -16,6 +16,7 @@ allCircularSplits <- function(k, labels = NULL) {
     tmp <- (1L:y) + x
     tmp %% (k + 1L) + tmp %/% (k + 1L)
   }
+
   k <- as.integer(k)
   l <- (k - 1L) %/% 2L
   res <- vector("list", k * (k - 1L) / 2)
@@ -32,8 +33,8 @@ allCircularSplits <- function(k, labels = NULL) {
       m <- k %/% 2
       res[(ind + 1):(ind + m)] <- lapply(0L:(m - 1L), fun, m)
     }
-
   }
+  res <- lapply(res, sort)
   if (is.null(labels)) labels <- as.character(1:k)
   attr(res, "labels") <- labels
   attr(res, "cycle") <- 1:k
@@ -42,14 +43,14 @@ allCircularSplits <- function(k, labels = NULL) {
 }
 
 
-getIndex <- function(left, right, n) {
-  if (n < max(left) | n < max(right)) stop("Error")
-  left <- as.integer(left)
-  right <- as.integer(right)
-  ll <- length(left)
-  lr <- length(right)
-  .C("giveIndex", left, right, ll, lr, as.integer(n), integer(ll * lr))[[6]] + 1
-}
+#getIndex <- function(left, right, n) {
+#  if (n < max(left) | n < max(right)) stop("Error")
+#  left <- as.integer(left)
+#  right <- as.integer(right)
+#  ll <- length(left)
+#  lr <- length(right)
+#  .C("giveIndex", left, right, ll, lr, as.integer(n), integer(ll * lr))[[6]] + 1
+#}
 
 
 splits2design <- function(obj, weight = NULL) {
@@ -70,26 +71,6 @@ splits2design <- function(obj, weight = NULL) {
 }
 
 
-hC <- function(g, set) {
-  intersec <- NULL
-  allEdges <- NULL
-  fromTo <- set
-  l <- length(set)
-  sptmp <- shortest_paths(g, fromTo[l], fromTo[1],
-                          output = c("epath"))$epath[[1]]
-  sptmp <- as.vector(sptmp)
-  allEdges <- sptmp
-  for (i in 2:length(set)) {
-    sptmp <- shortest_paths(g, fromTo[i - 1], fromTo[i],
-                            output = c("epath"))$epath[[1]]
-    sptmp <- as.vector(sptmp)
-    intersec <- c(intersec, intersect(allEdges, sptmp))
-    allEdges <- c(allEdges, sptmp)
-  }
-  #    allEdges = unique(allEdges)
-  list(allEdges, unique(allEdges), intersec)
-}
-
 
 addEdge <- function(network, desc, spl) {
   edge <- network$edge
@@ -97,7 +78,7 @@ addEdge <- function(network, desc, spl) {
   child <- edge[, 2]
   nTips <- length(network$tip.label)
 
-  desc2 <- SHORTwise(desc, nTips)
+  desc2 <- SHORTwise(desc) #, nTips)
   split <- desc2[spl]
 
   index <- network$splitIndex
@@ -174,7 +155,7 @@ circNetwork <- function(x, ord = NULL) {
   res <- stree(nTips, tip.label = attr(x, "labels"))
   res$edge[, 2] <- ord
   res$edge.length <- NULL
-  x <- SHORTwise(x, nTips)
+  x <- SHORTwise(x) #, nTips)
   spRes <- as.splits(res)[res$edge[, 2]]
   index <- match(spRes, x)
 
@@ -185,7 +166,7 @@ circNetwork <- function(x, ord = NULL) {
     index <- match(spRes, x)
   }
 
-  l <- lengths(oneWise(x, nTips))
+  l <- lengths(ONEwise(x))
   l2 <- lengths(x)
 
   #    dm <- as.matrix(compatible2(x))
@@ -293,7 +274,7 @@ circNetwork <- function(x, ord = NULL) {
 }
 
 
-#' Conversion among phylogenetic network 0bjects
+#' Conversion among phylogenetic network objects
 #'
 #' \code{as.networx} convert \code{splits} objects into a \code{networx}
 #' object. And most important there exists a generic \code{plot} function to
@@ -329,7 +310,7 @@ circNetwork <- function(x, ord = NULL) {
 #' tree1 <- rtree(20, rooted=FALSE)
 #' sp <- as.splits(rNNI(tree1, n=10))
 #' net <- as.networx(sp)
-#' plot(net, "2D")
+#' plot(net)
 #' \dontrun{
 #' # also see example in consensusNet
 #' example(consensusNet)
@@ -348,7 +329,6 @@ getOrdering <- function(x, opt=TRUE) {
   tree <- as.phylo(x, TRUE)
   tree <- reorder(tree)
   if(opt) tree <- optCycle(x, tree)
-#  tree <- as.phylo(x)
   nTips <- length(tree$tip.label)
   ord <- reorder(tree)$edge[, 2]
   ord <- ord[ord <= nTips]
@@ -367,7 +347,7 @@ addTrivialSplits <- function(obj) {
   STree <- stree(nTips, tip.label = attr(obj, "labels"))
   STree$edge.length <- NULL
   spRes <- as.splits(STree)[STree$edge[, 2]]
-  tmpIndex <- match(spRes, SHORTwise(obj, nTips))
+  tmpIndex <- match(spRes, SHORTwise(obj))
   if (any(is.na(tmpIndex))) {
     l.na <- sum(is.na(tmpIndex))
     obj <- c(obj, spRes[is.na(tmpIndex)])
@@ -397,7 +377,7 @@ as.networx.splits <- function(x, planar = FALSE, coord = c("none", "2D", "3D"),
   label <- attr(x, "label")
   x <- addTrivialSplits(x)
   nTips <- length(label)
-  x <- oneWise(x, nTips)
+  x <- ONEwise(x)
   l <- lengths(x)
   if (any(l == nTips)) x <- x[l != nTips] # get rid of trivial splits
   l <- lengths(x)
@@ -412,6 +392,8 @@ as.networx.splits <- function(x, planar = FALSE, coord = c("none", "2D", "3D"),
   }
   else c.ord <- getOrdering(x)
   attr(x, "cycle") <- c.ord
+  # check for star tree
+  if(length(x)==nTips) return(as.phylo(x))
   # which splits are in circular ordering
   circSplits <- which(countCycles(x, ord = c.ord) == 2)
   if (length(circSplits) == length(x)) planar <- TRUE
@@ -473,7 +455,6 @@ as.networx.phylo <- function(x, ...) {
 
 
 
-
 #' Computes a consensusNetwork from a list of trees Computes a \code{networx}
 #' object from a collection of splits.
 #'
@@ -503,17 +484,17 @@ as.networx.phylo <- function(x, ...) {
 #' bs <- bootstrap.phyDat(Laurasiatherian, FUN = function(x)nj(dist.hamming(x)),
 #'     bs=50)
 #' cnet <- consensusNet(bs, .3)
-#' plot(cnet, "2D")
+#' plot(cnet)
 #' \dontrun{
 #' library(rgl)
 #' open3d()
-#' plot(cnet, show.tip.label=FALSE, show.nodes=TRUE)
-#' plot(cnet, type = "2D", show.edge.label=TRUE)
+#' plot(cnet, type = "3D", show.tip.label=FALSE, show.nodes=TRUE)
+#' plot(cnet, type = "equal angle", show.edge.label=TRUE)
 #'
 #' tmpfile <- normalizePath(system.file("extdata/trees/RAxML_bootstrap.woodmouse", package="phangorn"))
 #' trees <- read.tree(tmpfile)
 #' cnet_woodmouse <- consensusNet(trees, .3)
-#' plot(cnet_woodmouse, type = "2D", show.edge.label=TRUE)
+#' plot(cnet_woodmouse, type = "equal angle", show.edge.label=TRUE)
 #' }
 #'
 #' @export consensusNet
@@ -544,9 +525,9 @@ createLabel <- function(x, y, label_y, type = "edge", nomatch = NA) {
   nTips <- length(tiplabel)
 
   spl_y <- changeOrder(spl_y, tiplabel)
-  spl_y <- SHORTwise(spl_y, nTips)
+  spl_y <- SHORTwise(spl_y)
 
-  ind <- match(SHORTwise(spl_x, nTips), spl_y)
+  ind <- match(SHORTwise(spl_x), spl_y)
   pos <-  which(!is.na(ind))
 
   res <- rep(nomatch, length(spl_x))
@@ -671,9 +652,10 @@ addConfidences.splits <- function(x, y, scaler = 1, ...) {
   }
 
   spl <- as.splits(y)
+# postprocess.proppart
   spl <- changeOrder(spl, tiplabel)
-  spl <- oneWise(spl, nTips)
-  ind <- match(oneWise(x, nTips), spl)
+  spl <- SHORTwise(spl)
+  ind <- match(SHORTwise(x), spl)
   #    pos <-  which(ind > nTips)
   pos <-  which(!is.na(ind))
   confidences <- rep(NA_real_, length(x)) # numeric(length(x))  #character
@@ -705,7 +687,7 @@ addConfidences.phylo <- function(x, y, ...) {
     as.is <- list(...)$as.is
   else as.is <- TRUE
   nTips <- length(x$tip.label)
-  spl <- as.splits(x) %>% oneWise(nTips = nTips)
+  spl <- as.splits(x) |> SHORTwise()
   conf <- attr(addConfidences(spl, y), "confidences")
   l <- lengths(spl)
   if (is.character(conf)) as.is <- TRUE
@@ -719,6 +701,15 @@ addConfidences.phylo <- function(x, y, ...) {
 }
 
 
+#' @export
+addConfidences.multiPhylo <- function(x, y, ...) {
+  x <- .uncompressTipLabel(x)
+  x <- unclass(x)
+  x <- lapply(x, addConfidences, y)
+  class(x) <- "multiPhylo"
+  x
+}
+
 #' @rdname addConfidences
 #' @export
 presenceAbsence <- function(x, y) {
@@ -775,7 +766,7 @@ spl2angle <- function(x) {
   if (!is.null(attr(x, "cycle"))) ord <- attr(x, "cycle")
   x <- changeOrder(x, attr(x, "labels")[ord])
   y <- lapply(x, function(x, l) (x - 1) / l * 360, l = l)
-  angle <- vapply(y, circ.mean, 0) %>% deg2rad()
+  angle <- vapply(y, circ.mean, 0) |> deg2rad()
   # angle <- ((vapply(x, sum, 0) / lengths(x) - 1) / l ) * 2*pi
   # kreis2kart(attr(x, "weight"), angle)
   angle
@@ -785,8 +776,8 @@ spl2angle <- function(x) {
 coords.equal.angle <- function(obj) {
   if (is.null(attr(obj, "order")) || (attr(obj, "order") == "postorder"))
     obj <- reorder.networx(obj)
-
   spl <- obj$splits
+  spl <- SHORTwise(spl) #, length(obj$tip.label))
   l <- length(obj$edge.length)
 #  ind1 <- which(!duplicated(obj$splitIndex))
   n <- max(obj$edge)
@@ -969,7 +960,7 @@ edgeLabels <- function(xx, yy, zz = NULL, edge) {
 #' tree1 <- rtree(20, rooted=FALSE)
 #' sp <- as.splits(rNNI(tree1, n=10))
 #' net <- as.networx(sp)
-#' plot(net, "2D")
+#' plot(net)
 #' \dontrun{
 #' # also see example in consensusNet
 #' example(consensusNet)
@@ -977,7 +968,7 @@ edgeLabels <- function(xx, yy, zz = NULL, edge) {
 #' @importFrom igraph graph_from_adjacency_matrix vcount topo_sort layout_nicely
 #' @method plot networx
 #' @export
-plot.networx <- function(x, type = "3D", use.edge.length = TRUE,
+plot.networx <- function(x, type = "equal angle", use.edge.length = TRUE,
                          show.tip.label = TRUE, show.edge.label = FALSE,
                          edge.label = NULL, show.node.label = FALSE,
                          node.label = NULL, show.nodes = FALSE,
@@ -988,8 +979,11 @@ plot.networx <- function(x, type = "3D", use.edge.length = TRUE,
                          cex.edge.label = cex, col.node.label = tip.color,
                          col.edge.label = tip.color, font.node.label = font,
                          font.edge.label = font, ...) {
-  type <- match.arg(type, c("3D", "2D", "equal angle"))
-  if (use.edge.length == FALSE) x$edge.length[] <- 1
+  type <- match.arg(type, c("equal angle", "3D", "2D"))
+  if (use.edge.length == FALSE){
+    x$edge.length[] <- 1
+    attr(x$splits, "weight") <- rep(1, length(x$splits))
+  }
   nTips <- length(x$tip.label)
   conf <- attr(x$splits, "confidences")
   index <- x$splitIndex
