@@ -1,360 +1,6 @@
 #
 # Data structures for ML and MP
 #
-fast.table <- function (data) {
-  if(!is.data.frame(data))
-    data <- as.data.frame(data, stringsAsFactors = FALSE)
-  da <- do.call("paste", c(data, sep = "\r"))
-  ind <- !duplicated(da)
-  levels <- da[ind]
-  cat <- factor(da, levels = levels)
-  nl <- length(levels(cat))
-  bin <- (as.integer(cat) - 1)
-  pd <- nl
-  bin <- bin[!is.na(bin)]
-  if (length(bin)) bin <- bin + 1
-  y <- tabulate(bin, pd)
-  result <- list(index = bin, weights = y, data = data[ind, ])
-  result
-}
-
-
-phyDat.default <- function (data, levels = NULL, return.index = TRUE,
-                  contrast = NULL, ambiguity = "?", compress=TRUE, ...){
-  if (is.matrix(data))
-    nam <- row.names(data)
-  else nam <- names(data)
-  if(is.null(nam))stop("data object must contain taxa names")
-  if(inherits(data, "character") | inherits(data, "numeric"))
-    data <- as.matrix(data)
-  if (inherits(data, "DNAbin"))
-    data <- as.character(data)
-  if (is.matrix(data))
-    data <- as.data.frame(t(data), stringsAsFactors = FALSE)
-  # new 4.4.2016 bug fix (reported by Eli Levy Karin)
-  #  if (is.vector(data) && !is.list(data))data = as.data.frame(data,
-  #  stringsAsFactors = FALSE)
-  else data <- as.data.frame(data, stringsAsFactors = FALSE)
-
-  if(length(data[[1]])==1) compress <- FALSE
-  if(compress){
-    ddd <- fast.table(data)
-    data <- ddd$data
-    weight <- ddd$weight
-    index <- ddd$index
-  }
-  else{
-    p <- length(data[[1]])
-    weight <- rep(1, p)
-    index <- 1:p
-  }
-  q <- length(data)
-  p <- length(data[[1]])
-#  tmp <- vector("list", q)
-  if (!is.null(contrast)) {
-    levels <- colnames(contrast)
-    all.levels <- rownames(contrast)
-    rownames(contrast) <- NULL
-  }
-  else {
-    if (is.null(levels))
-      stop("Either argument levels or contrast has to be supplied")
-    l <- length(levels)
-    contrast <- diag(l)
-    all.levels <- levels
-    if (!is.null(ambiguity)) {
-      all.levels <- c(all.levels, ambiguity)
-      k <- length(ambiguity)
-      if (k > 0)
-        contrast <- rbind(contrast, matrix(1, k, l))
-    }
-  }
-#  d <- dim(data)
-  att <- attributes(data)
-  data <- lapply(data, match, all.levels)
-  attributes(data) <- att
-
-  row.names(data) <- as.character(1:p)
-  data <- na.omit(data)
-  aaa <- match(index, attr(data, "na.action"))
-
-  if(!is.null(attr(data, "na.action"))) warning("Found unknown characters (not supplied in levels). Deleted sites with with unknown states.")
-
-  index <- index[is.na(aaa)]
-  index <- match(index, unique(index))
-  rn <- as.numeric(rownames(data))
-  attr(data, "na.action") <- NULL
-  weight <- weight[rn]
-  p <- dim(data)[1]
-  names(data) <- nam
-  attr(data, "row.names") <- NULL
-  attr(data, "weight") <- weight
-  attr(data, "nr") <- p
-  attr(data, "nc") <- length(levels)
-  if (return.index)
-    attr(data, "index") <- index
-  attr(data, "levels") <- levels
-  attr(data, "allLevels") <- all.levels
-  attr(data, "type") <- "USER"
-  attr(data, "contrast") <- contrast
-  class(data) <- "phyDat"
-  data
-}
-
-
-phyDat.DNA <- function (data, return.index = TRUE){
-  if (is.matrix(data))
-    nam <- row.names(data)
-  else nam <- names(data)
-  if (inherits(data,"DNAbin"))
-    data <- as.character(data)
-  if(inherits(data, "character")) data <- as.matrix(data)
-  if (is.matrix(data))
-    data <- as.data.frame(t(data), stringsAsFactors = FALSE)
-  else data <- as.data.frame(data, stringsAsFactors = FALSE)
-
-  data <- data.frame(tolower(as.matrix(data)), stringsAsFactors = FALSE)
-
-  ac <- c("a", "c", "g", "t", "u", "m", "r", "w", "s", "y",
-          "k", "v", "h", "d", "b", "n", "?", "-")
-  AC <- matrix(c(c(1, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 1, 1, 1),
-                 c(0, 1, 0, 0, 0, 1, 0, 0, 1, 1, 0, 1, 1, 0, 1, 1, 1, 1),
-                 c(0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 1, 1, 1, 1),
-                 c(0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 1, 1, 1, 1, 1, 1)),
-               18, 4, dimnames = list(NULL, c("a", "c", "g", "t")))
-
-  compress <- TRUE
-  if(length(data[[1]])==1) compress <- FALSE
-  if(compress){
-    ddd <- fast.table(data)
-    data <- ddd$data
-    weight <- ddd$weight
-    index <- ddd$index
-  }
-  else{
-    p <- length(data[[1]])
-    weight <- rep(1, p)
-    index <- 1:p
-  }
-#  q <- length(data)
-  p <- length(data[[1]])
-#  d <- dim(data)
-  att <- attributes(data)
-
-  data <- lapply(data, match, ac)
-#  data <- match(unlist(data), ac)
-#  attr(data, "dim") <- d
-#  data <- as.data.frame(data, stringsAsFactors=FALSE)
-  attributes(data) <- att
-  row.names(data) <- as.character(1:p)
-  data <- na.omit(data)
-  rn <- as.numeric(rownames(data))
-
-  if(!is.null(attr(data, "na.action"))) warning("Found unknown characters. Deleted sites with with unknown states.")
-
-  aaa <- match(index, attr(data, "na.action"))
-  index <- index[is.na(aaa)]
-  index <- match(index, unique(index))
-  rn <- as.numeric(rownames(data))
-  attr(data, "na.action") <- NULL
-
-  weight <- weight[rn]
-  p <- dim(data)[1]
-  names(data) <- nam
-  attr(data, "row.names") <- NULL
-  attr(data, "weight") <- weight
-  attr(data, "nr") <- p
-  attr(data, "nc") <- 4
-  if (return.index)
-    attr(data, "index") <- index
-  attr(data, "levels") <- c("a", "c", "g", "t")
-  attr(data, "allLevels") <- ac
-  attr(data, "type") <- "DNA"
-  attr(data, "contrast") <- AC
-  class(data) <- "phyDat"
-  data
-}
-
-
-phyDat.AA <- function (data, return.index = TRUE){
-  if(is.matrix(data)) nam <- row.names(data)
-  else nam <- names(data)
-  # AAbin
-  if (inherits(data,"AAbin"))
-    data <- as.character(data)
-  if(inherits(data, "character")) data <- as.matrix(data)
-  if (is.matrix(data))
-    data <- as.data.frame(t(data), stringsAsFactors = FALSE)
-  else data <- as.data.frame(data, stringsAsFactors = FALSE)
-
-  data <- data.frame(tolower(as.matrix(data)), stringsAsFactors = FALSE)
-
-  aa <- c("a", "r", "n", "d", "c", "q", "e", "g", "h", "i",
-    "l", "k", "m", "f", "p", "s", "t", "w", "y", "v")
-  aa2 <- c("a", "r", "n", "d", "c", "q", "e", "g", "h", "i",
-    "l", "k", "m", "f", "p", "s", "t", "w", "y", "v", "b",
-    "z", "x", "-", "?")
-  AA <- diag(20)
-  AA <- rbind(AA, matrix(0, 5, 20))
-  AA[21, 3] <- AA[21, 4] <- 1 # Aspartate or Asparagine
-  AA[22, 6] <- AA[22, 7] <- 1 #
-  AA[23:25, ] <- 1
-  dimnames(AA) <- list(aa2, aa)
-  compress <- TRUE
-  if(length(data[[1]])==1) compress <- FALSE
-  if(compress){
-    ddd <- fast.table(data)
-    data <- ddd$data
-    weight <- ddd$weight
-    index <- ddd$index
-    }
-  else{
-    p <- length(data[[1]])
-    weight <- rep(1, p)
-    index <- 1:p
-  }
-  q <- length(data)
-  p <- length(data[[1]])
-#  tmp <- vector("list", q)
-
-#  d <- dim(data)
-  att <- attributes(data)
-
-  data <- lapply(data, match, aa2)
-  # data <- match(unlist(data), aa2)
-  # attr(data, "dim") <- d
-  # data <- as.data.frame(data, stringsAsFactors=FALSE)
-  attributes(data) <- att
-
-  row.names(data) <- as.character(1:p)
-  data <- na.omit(data)
-  rn <- as.numeric(rownames(data))
-
-  if(!is.null(attr(data, "na.action"))) warning("Found unknown characters. Deleted sites with with unknown states.")
-
-  aaa <- match(index, attr(data, "na.action"))
-  index <- index[is.na(aaa)]
-  index <- match(index, unique(index))
-  rn <- as.numeric(rownames(data))
-  attr(data, "na.action") <- NULL
-  weight <- weight[rn]
-  p <- dim(data)[1]
-  names(data) <- nam
-  attr(data, "row.names") <- NULL
-  attr(data, "weight") <- weight
-  attr(data, "nr") <- p
-  attr(data, "nc") <- 20
-  if (return.index)
-    attr(data, "index") <- index
-  attr(data, "levels") <- aa
-  attr(data, "allLevels") <- aa2
-  attr(data, "type") <- "AA"
-  attr(data, "contrast") <- AA
-  class(data) <- "phyDat"
-  data
-}
-
-
-phyDat.codon <- function (data, return.index = TRUE, ambiguity = "---",
-                          NA_as_ambiguous=TRUE){
-  if(is.matrix(data)) nam <- row.names(data)
-  else nam <- names(data)
-  if (inherits(data,"DNAbin"))
-    data <- as.character(data)
-  if(inherits(data, "character")) data <- as.matrix(data)
-  if (is.matrix(data))
-    data <- as.data.frame(t(data), stringsAsFactors = FALSE)
-  else data <- as.data.frame(data, stringsAsFactors = FALSE)
-
-  data <- data.frame(tolower(as.matrix(data)), stringsAsFactors = FALSE)
-
-  data[data=="u"] <- "t"
-
-  splseq <- function (seq, frame = 0){
-    starts <- seq(from = frame + 1, to = length(seq), by = 3L)
-    sapply(starts, function(x) paste(seq[x:(x + 2L)], collapse=""))
-  }
-
-  data <- data.frame(lapply(data, splseq))
-  compress <- TRUE
-  if(nrow(data)==1) compress <- FALSE
-  if(compress){
-      ddd <- fast.table(data)
-      data <- ddd$data
-      weight <- ddd$weight
-      index <- ddd$index
-    }
-  else{
-    p <- length(data[[1]])
-    weight <- rep(1, p)
-    index <- 1:p
-  }
-  codon <- c("aaa", "aac", "aag", "aat", "aca", "acc", "acg", "act",
-      "aga", "agc", "agg", "agt", "ata", "atc", "atg", "att",
-      "caa", "cac", "cag", "cat", "cca", "ccc", "ccg", "cct", "cga",
-      "cgc", "cgg", "cgt", "cta", "ctc", "ctg", "ctt", "gaa", "gac",
-      "gag", "gat", "gca", "gcc", "gcg", "gct", "gga", "ggc", "ggg",
-      "ggt", "gta", "gtc", "gtg", "gtt", "tac", "tat",
-      "tca", "tcc", "tcg", "tct", "tgc", "tgg", "tgt", "tta",
-      "ttc", "ttg", "ttt")
-# ohne Stopcodons "taa", "tag", "tga",
-
-  CODON <- diag(61)
-
-  if(NA_as_ambiguous){
-    ambiguity <- unique(c("---", ambiguity))
-  }
-  if(ambiguity!=""){
-    codon_amb <- c(codon, ambiguity)
-    CODON <- rbind(CODON, matrix(1, length(ambiguity), 61))
-  }
-  else codon_amb <- codon
-  dimnames(CODON) <- list(codon_amb, codon)
-
-  q <- length(data)
-  p <- length(data[[1]])
-  tmp <- vector("list", q)
-
-  d <- dim(data)
-  att <- attributes(data)
-  data <- match(unlist(data), codon_amb)
-  if(NA_as_ambiguous){
-    ind <- match("---", codon_amb)
-    data[is.na(data)] <- ind
-  }
-  attr(data, "dim") <- d
-  data <- as.data.frame(data, stringsAsFactors=FALSE)
-  attributes(data) <- att
-
-  row.names(data) <- as.character(1:p)
-
-  data <- na.omit(data)
-  rn <- as.numeric(rownames(data))
-
-  if(!is.null(attr(data, "na.action"))) warning("Found unknown characters. Deleted sites with with unknown states.")
-
-  aaa <- match(index, attr(data, "na.action"))
-  index <- index[is.na(aaa)]
-  index <- match(index, unique(index))
-  rn <- as.numeric(rownames(data))
-  attr(data, "na.action") <- NULL
-
-  weight <- weight[rn]
-  p <- dim(data)[1]
-  names(data) <- nam
-  attr(data, "row.names") <- NULL
-  attr(data, "weight") <- weight
-  attr(data, "nr") <- p
-  attr(data, "nc") <- 61
-  if (return.index)
-    attr(data, "index") <- index
-  attr(data, "levels") <- codon
-  attr(data, "allLevels") <- codon_amb
-  attr(data, "type") <- "CODON"
-  attr(data, "contrast") <- CODON
-  class(data) <- "phyDat"
-  data
-}
 
 
 #' Conversion among Sequence Formats
@@ -372,13 +18,6 @@ phyDat.codon <- function (data, return.index = TRUE, ambiguity = "---",
 #' in simulation studies. For further details see the vignette
 #' phangorn-specials.
 #'
-#' \code{write.phyDat} calls the function write.dna or write.nexus.data and
-#' \code{read.phyDat} calls the function \code{read.dna}, \code{read.aa} or
-#' \code{read.nexus.data} see for more details over there.
-#'
-#' You may import data directly with \code{\link[ape]{read.dna}} or
-#' \code{\link[ape]{read.nexus.data}} and convert the data to class phyDat.
-#'
 #' The generic function \code{c} can be used to to combine sequences and
 #' \code{unique} to get all unique sequences or unique haplotypes.
 #'
@@ -394,15 +33,6 @@ phyDat.codon <- function (data, return.index = TRUE, ambiguity = "---",
 #' @param type Type of sequences ("DNA", "AA", "CODON" or "USER").
 #' @param levels Level attributes.
 #' @param return.index If TRUE returns a index of the site patterns.
-#' @param file A file name.
-#' @param format File format of the sequence alignment (see details).  Several
-#' popular formats are supported: "phylip", "interleaved", "sequential",
-#' "clustal", "fasta" or "nexus", or any unambiguous abbreviation of these.
-#' @param colsep a character used to separate the columns (a single space by
-#' default).
-#' @param nbcol a numeric specifying the number of columns per row (-1 by
-#' default); may be negative implying that the nucleotides are printed on a
-#' single line.
 #' @param n Number of sequences.
 #' @param names Names of sequences.
 #' @param subset a subset of taxa.
@@ -422,9 +52,6 @@ phyDat.codon <- function (data, return.index = TRUE, ambiguity = "---",
 #' (happens frequently with ambiguous sites).
 #' @param ambiguity character for ambiguous character and no contrast is
 #' provided.
-#' @param codonstart an integer giving where to start the translation. This
-#' should be 1, 2, or 3, but larger values are accepted and have for effect to
-#' start the translation further within the sequence.
 #' @param ... further arguments passed to or from other methods.
 #' @return The functions return an object of class \code{phyDat}.
 #' @author Klaus Schliep \email{klaus.schliep@@gmail.com}
@@ -448,6 +75,10 @@ phyDat.codon <- function (data, return.index = TRUE, ambiguity = "---",
 #' subset(Laurasiatherian, subset=1:5)
 #' # the first 5 characters
 #' subset(Laurasiatherian, select=1:5, site.pattern = FALSE)
+#' # subsetting with []
+#' Laurasiatherian[1:5, 1:20]
+#' # short for
+#' subset(Laurasiatherian, subset=1:5, select=1:20, site.pattern = FALSE)
 #' # the first 5 site patterns (often more than 5 characters)
 #' subset(Laurasiatherian, select=1:5, site.pattern = TRUE)
 #' # transform into old ape format
@@ -462,7 +93,7 @@ phyDat.codon <- function (data, return.index = TRUE, ambiguity = "---",
 #' @rdname phyDat
 #' @export
 phyDat <- function (data, type="DNA", levels=NULL, return.index = TRUE, ...){
-  if (inherits(data, "DNAbin")) type <- "DNA"
+#  if (inherits(data, "DNAbin")) type <- "DNA"
   pt <- match.arg(type, c("DNA", "AA", "CODON", "USER"))
   if(pt=="DNA") dat <- phyDat.DNA(data, return.index=return.index, ...)
   if(pt=="AA") dat <- phyDat.AA(data, return.index=return.index, ...)
@@ -473,33 +104,6 @@ phyDat <- function (data, type="DNA", levels=NULL, return.index = TRUE, ...){
 }
 
 
-#' @rdname phyDat
-#' @export
-dna2codon <- function(x, codonstart=1, ambiguity="---", ...){
-  if(!inherits(x, "phyDat"))stop("x needs to be of class phyDat!")
-  if(attr(x, "type")=="AA")stop("x needs to be a nucleotide sequence!")
-
-  if(codonstart>1){
-    del <- -seq_len(codonstart)
-    x <- subset(x, select=del, site.pattern=FALSE)
-  }
-  n_sites <- sum(attr(x, "weight"))
-  if( (n_sites %% 3) ){
-    keep <- seq_len( (n_sites %/% 3) * 3 )
-    x <- subset(x, select=keep, site.pattern=FALSE)
-  }
-  phyDat.codon(as.character(x), ambiguity=ambiguity, ...)
-}
-
-
-#' @rdname phyDat
-#' @export
-codon2dna <- function(x){
-  if(!inherits(x, "phyDat"))stop("x needs to be of class phyDat!")
-  phyDat.DNA(as.character(x))
-}
-
-
 #' @rdname phyDat
 #' @export
 as.phyDat <- function (x, ...){
@@ -643,7 +247,6 @@ acgt2ry <- function(obj){
 
 #' @rdname phyDat
 #' @export
-# replace as.character.phyDat weniger Zeilen, works also for codons
 as.character.phyDat <- function (x, allLevels=TRUE, ...){
   nr <- attr(x, "nr")
 #  nc <- attr(x, "nc")
@@ -761,48 +364,35 @@ print.phyDat <- function (x, ...){
 }
 
 
-# in C++ to replace aggregate or use of distinct from dplyr / data.table
-aggr <- function(weight, ind){
-  res <- numeric(max(ind))
-  for(i in seq_along(weight))
-    res[ind[i]] <- res[ind[i]] + weight[i]
-  res
+## @export
+summary.phyDat <- function (x, ...){
+  nc  <- attr(x, "nc")
+  nseq <- length(x)
+  nchar <- sum(attr(x,"weight"))
+  unique_sites <- attr(x, "nr")
+  tmp <- logical(unique_sites)
+  for(i in 2:nseq) tmp <- tmp | (x[[1]] != x[[i]])
+  const_sites <- sum(attr(x, "weight")[tmp==0])
+  list(nseq=nseq, nchar=nchar, unique_sites=unique_sites,
+       const_sites=const_sites)
 }
 
 
-# data has to be a data.frame in cbind.phyDat
-fast.table2 <- function (data){
-  if(!is.data.frame(data))
-    data <- as.data.frame(data, stringsAsFactors = FALSE)
-  da <- do.call("paste", data)
-  ind <- !duplicated(da)
-  levels <- da[ind]
-  cat <- factor(da, levels = levels)
-#  nl <- length(levels)
-  bin <- (as.integer(cat) - 1L)
-  bin <- bin[!is.na(bin)]
-  if (length(bin)) bin <- bin + 1L
-  result <- list(index = bin, pos = ind)
-  result
-}
-
-
-# @rdname phyDat
 #' @export cbind.phyDat
 #' @export
 cbind.phyDat <- function(..., gaps="-", compress=TRUE){
   object <- as.list(substitute(list(...)))[-1]
   x <- list(...)
   n <- length(x)
-  if (n == 1)
-    return(x[[1]])
-#  type <- attr(x[[1]], "type")
+  if (n == 1) return(x[[1]])
+
+  types <- sapply(x, function(x)attr(x, "type"))
+# if(length(unique(types))>1) stop("All alignments need to have the same type!")
+  #  type <- attr(x[[1]], "type")
   nr <- numeric(n)
-
   ATTR <- attributes(x[[1]])
-
   nr[1] <- sum(attr(x[[1]], "weight"))
-#  levels <- attr(x[[1]], "levels")
+  #  levels <- attr(x[[1]], "levels")
   allLevels <- attr(x[[1]], "allLevels")
   gapsInd <- match(gaps, allLevels)
   snames <- vector("list", n)
@@ -822,14 +412,11 @@ cbind.phyDat <- function(..., gaps="-", compress=TRUE){
   wvec <- cumsum(wvec)
   snames <- unique(unlist(snames))
   weight <- numeric(vec[n+1])
-
   index <- numeric(wvec[n+1])
-
   ATTR$names <- snames
   ATTR$nr <- vec[n+1]
-
   tmp <- matrix(gapsInd, vec[n+1], length(snames),
-          dimnames = list(NULL, snames))
+                dimnames = list(NULL, snames))
   tmp <- as.data.frame(tmp)
   add.index <- TRUE
   for(i in 1:n){
@@ -838,9 +425,11 @@ cbind.phyDat <- function(..., gaps="-", compress=TRUE){
     weight[(vec[i]+1):vec[i+1]] <- attr(x[[i]], "weight")
   }
   if(compress){
-    ddd <- fast.table2(tmp)
-    tmp <- tmp[ddd$pos, ]
-    weight <- aggregate(weight, by=list(ddd$index), FUN=sum)$x
+    dindex <- grp_duplicated(as.matrix(tmp), MARGIN=1)
+    attr(dindex, "nlevels") <- NULL
+    weight <- aggregate(weight, by=list(dindex), FUN=sum)$x
+    pos <- which(!duplicated(dindex))
+    tmp <- tmp[pos,]
   }
   if(any(sapply(x, function(x)is.null(attr(x, "index"))))) add.index <- FALSE
   if(add.index  & compress){
@@ -848,9 +437,9 @@ cbind.phyDat <- function(..., gaps="-", compress=TRUE){
       tmp2 <- attr(x[[i]], "index")
       if(!is.null(tmp2)){
         if(is.data.frame(tmp2))index[(wvec[i]+1):wvec[i+1]] <-
-            ddd$index[(vec[i]+1):vec[i+1]][tmp2[,1]]
+            dindex[(vec[i]+1):vec[i+1]][tmp2[,1]]
         else index[(wvec[i]+1):wvec[i+1]] <-
-                ddd$index[(vec[i]+1):vec[i+1]][tmp2]
+            dindex[(vec[i]+1):vec[i+1]][tmp2]
       }
       else add.index <- FALSE
     }
@@ -862,97 +451,13 @@ cbind.phyDat <- function(..., gaps="-", compress=TRUE){
   tmp
 }
 
+
 # @rdname phyDat
 #' @export c.phyDat
 #' @export
 c.phyDat <- cbind.phyDat
 
 
-#' @rdname phyDat
-#' @export
-write.phyDat <- function(x, file, format="phylip", colsep = "", nbcol=-1, ...){
-  formats <- c("phylip", "nexus", "interleaved", "sequential", "fasta")
-  format <- match.arg(tolower(format), formats)
-  if(format=="nexus"){
-    type <- attr(x, "type")
-    if(type=="DNA") write.nexus.data(as.list(as.data.frame(x)), file,
-                                     format = "dna",...)
-    else write.nexus.data(as.list(as.data.frame(x)), file,
-                          format = "protein", ...)
-  }
-  else{
-    if(format=="phylip") format <- "interleaved"
-    write.dna(as.character(x), file, format=format, colsep = colsep,
-              nbcol=nbcol, ...)
-  }
-}
-
-
-
-read.fasta.user <- function (file, skip = 0, nlines = 0,
-                             comment.char = "#", seq.names = NULL){
-  getTaxaNames <- function(x) {
-    x <- sub("^ +", "", x)
-    x <- sub(" +$", "", x)
-    x <- sub("^['\"]", "", x)
-    x <- sub("['\"]$", "", x)
-    x
-  }
-
-  X <- scan(file = file, what = character(), sep = "\n", quiet = TRUE,
-            skip = skip, nlines = nlines, comment.char = comment.char)
-
-  start <- grep("^ {0,}>", X)
-  taxa <- X[start]
-  n <- length(taxa)
-  obj <- vector("list", n)
-  if (is.null(seq.names)) {
-    taxa <- sub("^ {0,}>", "", taxa)
-    seq.names <- getTaxaNames(taxa)
-  }
-  start <- c(start, length(X) + 1)
-  for (i in 1:n) obj[[i]] <- unlist(strsplit(gsub(" ",
-                      "", X[(start[i] + 1):(start[i + 1] - 1)]), NULL))
-  names(obj) <- seq.names
-  obj <- lapply(obj, tolower)
-  obj
-}
-
-
-#' @rdname phyDat
-#' @export
-read.phyDat <- function(file, format="phylip", type="DNA", ...){
-
-  formats <- c("phylip", "nexus", "interleaved", "sequential", "fasta",
-               "clustal")
-  format <- match.arg(tolower(format), formats)
-
-  if(format=="nexus") data <- read.nexus.data(file, ...)
-  else {
-    if(format=="phylip") format <- "sequential" #"interleaved"
-    if (type == "DNA" || type == "CODON"){
-      data <- read.dna(file, format, as.character = TRUE, ...)
-    }
-    if (type == "AA") data <- read.aa(file, format=format, ...)
-    if (type == "USER"){
-      if(format=="fasta")
-        data <- read.fasta.user(file)
-      else data <- read.dna(file, format, as.character = TRUE)
-      extras <- match.call(expand.dots = FALSE)$...
-      extras <- lapply(extras, eval)
-      return(phyDat(data, type, levels=extras$levels,
-              ambiguity = extras$ambiguity, contrast = extras$contrast))
-    }
-    # raus
-  }
-  if(is.list(data)){
-    ll <- lengths(data)
-    if(!all(ll == ll[[1]])) stop("sequences have different length")
-  }
-  phyDat(data, type, return.index = TRUE)
-}
-
-
 #' @rdname phyDat
 #' @export
 baseFreq <- function(obj, freq=FALSE, all=FALSE, drop.unused.levels = FALSE){
@@ -978,14 +483,43 @@ phylo <- function(edge, tip, edge.length=NULL){
   res
   }
 
-
-getCols <- function (data, cols){
+compress.phyDat <- function(data){
   attrib <- attributes(data)
   attr(data, "class") <- "list"
+  index <- grp_duplicated( matrix(unlist(data, use.names = FALSE), attrib$nr,
+                                  length(data)))
+  attrib$nr <- attr(index, "nlevels")
+  attr(index, "nlevels") <- NULL
+  pos <- which(!duplicated(index))
+  attrib$weight <- tapply(attrib$weight, index, sum)
+  if(is.null(attrib$index)) attrib$index <-index
+  else attrib$index <- index[attrib$index]
+  for(i in seq_len(length(data))) data[[i]] <- data[[i]][pos]
+  attributes(data) <- attrib
+  attr(data, "class") <- "phyDat"
+  data
+}
+
+
+getCols <- function (data, cols, compress=FALSE){
+  attrib <- attributes(data)
+  if(inherits(attr(data, "index"), "data.frame")) compress <- FALSE
+  attr(data, "class") <- "list"
   data <- data[cols]
   if (is.character(cols))
     attrib$names <- cols
   else attrib$names <- attrib$names[cols]
+  if(compress){
+    index <- grp_duplicated( matrix(unlist(data, use.names = FALSE), attrib$nr,
+                                    length(data)))
+    attrib$nr <- attr(index, "nlevels")
+    attr(index, "nlevels") <- NULL
+    pos <- which(!duplicated(index))
+    attrib$weight <- tapply(attrib$weight, index, sum)
+    if(is.null(attrib$index)) attrib$index <-index
+    else attrib$index <- index[attrib$index]
+    for(i in seq_len(length(data))) data[[i]] <- data[[i]][pos]
+  }
   attributes(data) <- attrib
   attr(data, "class") <- "phyDat"
   data
@@ -996,13 +530,13 @@ getRows <- function (data, rows, site.pattern = TRUE){
   index <- attr(data, "index")
   if(is.data.frame(index))index <- index[,1]
   if(!site.pattern){
+    if(is.null(index)) index <- seq_len(length(data[[1]]))
     weight <- tabulate(index[rows])
     ind <- which(weight>0)
 # update index
     new_index <- integer(length(weight))
     new_index[ind] <- seq_along(ind)
     attr(data, "index") <- new_index[index[rows]]
-
     rows <- ind   # rows[ind]
     weight <- weight[ind]
   }
@@ -1021,16 +555,22 @@ getRows <- function (data, rows, site.pattern = TRUE){
 #' @rdname phyDat
 #' @method subset phyDat
 #' @export
-subset.phyDat <- function (x, subset, select, site.pattern = TRUE,...){
-  if (!missing(subset)) x <- getCols(x, subset)
+subset.phyDat <- function (x, subset, select, site.pattern = TRUE, ...){
+  if (!missing(subset)){
+    if(is.numeric(subset) & any(subset>length(x))) stop("subscript out of bounds")
+    x <- getCols(x, subset, ...)
+  }
   if (!missing(select)){
-     if(any(is.na(select))) return(NULL)
-     x <- getRows(x, select, site.pattern=site.pattern)
+    w <- attr(x, "weight")
+    if(site.pattern) if(any(select > length(w))) stop("subscript out of bounds")
+    else if(any(select > sum(w))) stop("subscript out of bounds")
+    if(any(is.na(select))) return(NULL)
+    x <- getRows(x, select, site.pattern=site.pattern)
   }
   x
 }
 
-## Needs testing that it is not used e.g. prepareDataFitch returns no class
+
 #' @param i,j	indices of the rows and/or columns to select or to drop. They
 #' may be numeric, logical, or character (in the same way than for standard R
 #' objects).
@@ -1038,46 +578,74 @@ subset.phyDat <- function (x, subset, select, site.pattern = TRUE,...){
 #' @rdname phyDat
 #' @export
 "[.phyDat" <- function(x, i, j, ..., drop=FALSE){
-   subset(x, subset = i, select = j, site.pattern=FALSE)
+   subset(x, subset = i, select = j, site.pattern=FALSE, compress=FALSE)
 }
 
 
 #' @rdname phangorn-internal
+#' @importFrom fastmatch fmatch
 #' @export
-map_duplicates <-  function(x, dist=TRUE, ...){
+map_duplicates <-  function(x, dist=length(x)<500, ...){
   labels <- names(x)
+  nr <- attr(x, "nr")
+
   if(dist){
     y <- as.matrix(dist.hamming(x, FALSE))
     l <- nrow(y)
-    z <- character(l)
-    for(i in seq_len(l)) z[i] <- paste( round(y[i, ], 8), collapse="_")
-    ind <- duplicated(z)
+#    z <- character(l)
+#    for(i in seq_len(l)) z[i] <- paste( round(y[i, ], 8), collapse="_")
+    ind <- duplicated(y)
   }
   else ind <- duplicated(x)
   res <- NULL
   if(any(ind)){
-    if(dist) ind2 <- match(z[ind], z)
-    else ind2 <- match(x[ind], x)
-    res <- data.frame(duplicates=labels[ind], where=labels[ind2],
+    ind2 <- grp_duplicated( matrix(unlist(x, recursive = FALSE,
+                            use.names = FALSE), nr, length(labels)), MARGIN=2)
+    if(dist) ind2 <- grp_duplicated(y)
+    ind2 <- ind2[ind]
+    res <- data.frame(duplicates=labels[ind], where=labels[!ind][ind2],
                       stringsAsFactors = FALSE)
   }
   res
 }
 
 
-#duplicated_phyDat <- function(x, ...){
-#    tmp <- map_duplicates(x)[,1]
-#    getCols(x, setdiff(names(x), tmp))
-#}
-
-
 #' @rdname phyDat
 #' @method unique phyDat
 #' @export
 unique.phyDat <- function(x, incomparables=FALSE, identical=TRUE, ...){
-  if(identical) return(getCols(x, !duplicated(x)))
-  tmp <- map_duplicates(x)[,1]
-  getCols(x, setdiff(names(x), tmp))
+  tmp <- map_duplicates(x, !identical)
+  if (!is.null(tmp)) {
+    x <- getCols(x, setdiff(names(x), tmp[, 1]))
+    attr(x, "duplicated") <- list(tmp)
+  }
+  x
+}
+
+
+addTaxa <- function(tree, dup_list) {
+  fun <- function(x, dup_list){
+    if(Ntip(x)==1L){
+      dup <- dup_list[[1]][,1]
+      x <- add.tips(x, dup, rep(2, length(dup)))
+      dup_list[[1]] <- NULL
+    }
+    for (i in seq_along(dup_list)) {
+      dup <- dup_list[[i]]
+      x <- add.tips(x, dup[, 1], dup[, 2])
+    }
+    x
+  }
+  if(inherits(tree, "phylo")) return(fun(tree, dup_list))
+  if(inherits(tree, "multiPhylo")){
+    trees <- .uncompressTipLabel(tree)
+    trees <- unclass(trees)
+    trees <- lapply(trees, fun, dup_list)
+    class(trees) <- "multiPhylo"
+    trees <- .compressTipLabel(trees)
+    return(trees)
+  }
+  NULL
 }
 
 
@@ -1098,16 +666,45 @@ removeUndeterminedSites <- function(x, ...){
 }
 
 
-removeParsUninfoSites <- function(data){
+removeParsUninfoSites <- function(data, exact=TRUE){
   nr <- attr(data, "nr")
-  pis <- parsinfo(data)
+  pis <- parsinfo(data, exact)
   if (length(pis) > 0){
     p0 <- sum(attr(data, "weight")[pis[, 1]] * pis[, 2])
     data <- getRows(data, c(1:nr)[-pis[, 1]], TRUE)
+    if(is.null(attr(data, "informative")))
+      attr(data, "informative") <- seq_len(nr)[-pis[, 1]]
+    else attr(data, "informative") <- attr(data, "informative")[-pis[, 1]]
   }
   else p0 <- 0
   if(length(attr(data, "p0"))) p0 <- p0 + attr(data, "p0")
-  attr(data, "p0") <-  p0
+  attr(data, "p0") <- p0
+  data
+}
+
+
+removeParsimonyUninfomativeSites <- function(data, recursive=FALSE, exact=TRUE){
+  dup_list <- NULL
+  tmp <- TRUE
+  star_tree <- FALSE
+  while (tmp) {
+    nam <- names(data)
+    data <- removeParsUninfoSites(data, exact)
+    if(!recursive) return(data)
+    if (attr(data, "nr") == 0) {
+      star_tree <- TRUE
+      break()
+      tmp <- FALSE
+    }
+    # unique sequences
+    dup <- map_duplicates(data)
+    if (!is.null(dup)) {
+      dup_list <- c(list(dup), dup_list)
+      data <- subset(data, setdiff(names(data), dup[, 1]))
+    }
+    else break() # tmp <- FALSE
+  }
+  attr(data, "duplicated") <- dup_list
   data
 }
 
@@ -1116,23 +713,31 @@ removeParsUninfoSites <- function(data){
 #' @export
 allSitePattern <- function(n, levels=c("a", "c", "g", "t"), names=NULL){
   l <- length(levels)
-  X <- vector("list", n)
-  if(is.null(names))names(X) <- paste0("t", 1:n)
-  else names(X) <- names
+  X <- matrix(NA_integer_, n, l^n)
+  if(is.null(names))rownames(X) <- paste0("t", 1:n)
+  else rownames(X) <- names
   for(i in 1:n)
-    X[[i]] <- rep(rep(levels, each=l^(i-1)), l^(n-i))
-  X <- as.data.frame(X)
+    X[i, ] <- rep(rep(levels, each=l^(i-1)), l^(n-i))
   phyDat.default(X, levels, compress=FALSE, return.index=FALSE)
 }
 
 
-constSitePattern <- function(n, levels=c("a", "c", "g", "t"), names=NULL){
-  l <- length(levels)
+##constSitePattern <- function(n, levels=c("a", "c", "g", "t"), names=NULL){
+constSitePattern <- function(n, names=NULL, type="DNA", levels=NULL){
+  if(type=="DNA"){
+    levels <- c("a", "c", "g", "t")
+    l <- 4L
+  } else if(type=="AA"){
+    levels <- c("A", "R", "N", "D", "C", "Q", "E", "G", "H", "I", "L", "K", "M",
+                "F", "P", "S", "T", "W", "Y", "V")
+    l <- 20L
+  }
+  else l <- length(levels)
   X <- matrix(0, l, n)
   X <- matrix(rep(levels, each=n), n, l)
-  if(is.null(names))rownames(X) <- paste0("t", 1:n)
+  if(is.null(names)) rownames(X) <- paste0("t", seq_len(n))
   else rownames(X) <- names
-  phyDat.default(X, levels)
+  phyDat(X, type=type, levels=levels)
 }
 
 
@@ -1162,10 +767,11 @@ constSitePattern <- function(n, levels=c("a", "c", "g", "t"), names=NULL){
 #' @seealso \code{\link[ape]{read.dna}}, \code{\link[ape]{read.GenBank}},
 #' \code{\link[phangorn]{phyDat}}, \code{\link[seqinr]{read.alignment}}
 #' @references % Anonymous. FASTA format description. %
-#' \url{https://www.ncbi.nlm.nih.gov/blast/fasta.shtml} Felsenstein, J. (1993)
-#' Phylip (Phylogeny Inference Package) version 3.5c. Department of Genetics,
-#' University of Washington.
-#' \url{http://evolution.genetics.washington.edu/phylip/phylip.html}
+#' \url{https://en.wikipedia.org/wiki/FASTA_format}
+#'
+#' Felsenstein, J. (1993) Phylip (Phylogeny Inference Package) version 3.5c.
+#' Department of Genetics, University of Washington.
+#' \url{https://evolution.genetics.washington.edu/phylip/phylip.html}
 #' @keywords IO
 #' @export read.aa
 read.aa <- function (file, format = "interleaved", skip = 0, nlines = 0,
