@@ -5,7 +5,7 @@
 #'
 #' \code{read.nexus.splits}, \code{write.nexus.splits}, \code{read.nexus.networx}, \code{write.nexus.networx}
 #' can be used to import and export splits and networks with nexus format
-#' and allow to exchange these object with other software like Splitstree.
+#' and allow to exchange these object with other software like SplitsTree.
 #' \code{write.splits} returns a human readable output.
 #'
 #' @param file a file name.
@@ -22,7 +22,7 @@
 #' @param labels names of taxa.
 #' @return \code{write.nexus.splits} and \code{write.nexus.networx} write out
 #' the \code{splits} and \code{networx} object to read with
-#' other software like Splitstree.
+#' other software like SplitsTree.
 #' \code{read.nexus.splits} and \code{read.nexus.networx} return an \code{splits}
 #' and \code{networx} object.
 #' @note \code{read.nexus.splits} reads in the splits block of a nexus file. It
@@ -94,7 +94,7 @@ read.nexus.splits <- function(file) {
     tmp <- sub("\\s+", "", tmp)
     tmp <- strsplit(tmp, "\t")[[1]]
     if (flab) {
-      labels[j] <- gsub("'", "", tmp[ind[1]]) %>% as.numeric
+      labels[j] <- gsub("'", "", tmp[ind[1]]) |> as.numeric()
     }
     if (fwei) weights[j] <- as.numeric(tmp[ind[2]])
     if (fcon) confidences[j] <- as.numeric(tmp[ind[3]])
@@ -129,7 +129,7 @@ write.nexus.splits <- function(obj, file = "", weights = NULL, taxa = TRUE,
                                append = FALSE) {
   taxa.labels <- attr(obj, "labels")
   ntaxa <- length(taxa.labels)
-  obj <- oneWise(obj, ntaxa)
+  obj <- ONEwise(obj)
   ind <- which(lengths(obj) == ntaxa)
   if (length(ind)) obj <- obj[-ind]
   nsplits <- length(obj)
@@ -138,7 +138,7 @@ write.nexus.splits <- function(obj, file = "", weights = NULL, taxa = TRUE,
   else fwei <- TRUE
   if (!append) {
     cat("#NEXUS\n\n", file = file)
-    cat("[Splits block for Spectronet or Splitstree]\n", file = file,
+    cat("[Splits block for Spectronet or SplitsTree]\n", file = file,
       append = TRUE)
     cat("[generated by phangorn", packageDescription("phangorn",
       fields = "Version"), "]\n\n", file = file, append = TRUE)
@@ -203,7 +203,7 @@ write.nexus.networx <- function(obj, file = "", taxa = TRUE, splits = TRUE,
                                 append = FALSE) {
   if (!append) {
     cat("#NEXUS\n\n", file = file)
-    cat("[Network block for Spectronet or Splitstree]\n", file = file,
+    cat("[Network block for Spectronet or SplitsTree]\n", file = file,
       append = TRUE)
     cat("[generated by phangorn", packageDescription("phangorn",
       fields = "Version"), "]\n\n", file = file, append = TRUE)
