@@ -20,16 +20,6 @@ getRoot <- function(tree) {
 }
 
 
-#reroot2 <- function(tree, node) {
-#  if (node == getRoot(tree)) return(tree)
-#  anc <- Ancestors(tree, node, "all")
-#  l <- length(anc)
-#  ind <- match(c(node, anc[-l]), tree$edge[, 2])
-#  tree$edge[ind, c(1, 2)] <- tree$edge[ind, c(2, 1)]
-#  reorderPruning(tree)
-#}
-
-# seems to work now
 reroot <- function (tree, node, switch_root=TRUE) {
   root <- getRoot(tree)
   if (node == root)
@@ -86,10 +76,10 @@ changeEdgeLength <- function(tree, edge, edge.length) {
 #' so far does not transform node.labels properly.
 #'
 
-#' @param tree an object of class \code{phylo}
+#' @param tree an object of class \code{phylo}.
 #' @param FUN a function evaluated on the nodelabels, result must be logical.
-#' @param node.labels are nodel labels 'support' values, 'label' or should be
-#' 'deleted'
+#' @param node.labels are node labels 'support' values (edges), 'label' or
+#' should labels get 'deleted'?
 #' @param \dots further arguments, passed to other methods.
 #' @return \code{pruneTree} and \code{midpoint} a tree. \code{getRoot} returns
 #' the root node.
@@ -105,10 +95,12 @@ changeEdgeLength <- function(tree, edge, edge.length) {
 #' tree2 <- midpoint(tree)
 #' tree3 <- pruneTree(tree, .5)
 #'
+#' old.par <- par(no.readonly = TRUE)
 #' par(mfrow = c(3,1))
 #' plot(tree, show.node.label=TRUE)
 #' plot(tree2, show.node.label=TRUE)
 #' plot(tree3, show.node.label=TRUE)
+#' par(old.par)
 #'
 #' @rdname midpoint
 #' @export midpoint
@@ -162,7 +154,7 @@ midpoint.phylo <- function(tree, node.labels = "support", ...) {
   tree$edge.length[tree$edge[, 2] == ind] <- maxdm
   ind <- c(ind, which.max(maxD1))
   maxdm <- maxdm + maxD1[ind[2]]
-  rn <- max(tree$edge) + 1
+  rn <- max(tree$edge) + 1L
   edge <- tree$edge
   el <- tree$edge.length
   children <- tree$edge[, 2]
@@ -186,8 +178,9 @@ midpoint.phylo <- function(tree, node.labels = "support", ...) {
     el[right[i]] <- eltmp
   }
   tree$edge.length <- el
+  storage.mode(edge) <- "integer"
   tree$edge <- edge
-  tree$Nnode <- tree$Nnode + 1
+  tree$Nnode <- tree$Nnode + 1L
   attr(tree, "order") <- NULL
   tree <- reroot(tree, rn)
   if (!is.null(tree$node.label)) {
@@ -232,7 +225,7 @@ pruneTree <- function(tree, ..., FUN = ">=") {
   bs <- rep(TRUE, m)
   bs[ (nTips + 1):m] <- sapply(as.numeric(as.character(tree$node)), FUN, ...)
   tree$edge.length[!bs[tree$edge[, 2]]] <- 0
-
+  attr(tree, "order") <- NULL
   reorder(di2multi(tree), "postorder")
 }
 
@@ -320,6 +313,7 @@ dropNode <- function(x, i, check.binary = FALSE, check.root = TRUE,
   else edge <- edge[-ch, ]
   if (nrow(edge) < 3) return(NULL)
   ind <- which(edge[, 1] == pa)
+  sibs <- edge[ind, 2L]
   if (root == pa) {
     if (length(ind) == 1) {
       edge <- edge[-ind, ]
@@ -332,7 +326,7 @@ dropNode <- function(x, i, check.binary = FALSE, check.root = TRUE,
       if (newedge[1] == newroot) edge[n - 1, ] <- newedge
       else edge[n - 1, ] <- newedge[2:1]
       edge <- edge[-n, ]
-      x$Nnode <- length(unique(edge[, 1]))
+      x$Nnode <- as.integer(length(unique(edge[, 1])))
       edge[edge == newroot] <- root
       pa <- newroot
     }
@@ -344,17 +338,17 @@ dropNode <- function(x, i, check.binary = FALSE, check.root = TRUE,
     if (length(ind) == 1) {
       edge[nind, 2] <- edge[ind, 2]
       edge <- edge[-ind, ]
-      x$Nnode <- length(unique(edge[, 1]))
+      x$Nnode <- as.integer(length(unique(edge[, 1])))
     }
   }
   x$edge <- edge
   y <- x
   y$edge <- edge2
-  y$Nnode <- length(unique(edge2[, 1]))
-  list(x, y, pa)
+  y$Nnode <- as.integer(length(unique(edge2[, 1])))
+  list(x, y, pa, sibs)
 }
 
-
+# nur mit edge matrix
 # postorder remained tip in 1:nTips
 addOne <- function(tree, tip, i) {
   edge <- tree$edge
@@ -369,11 +363,11 @@ addOne <- function(tree, tip, i) {
   else edge <- rbind(edge[1:(ind - 1), ], matrix(c(m, m, k, tip), 2, 2),
       edge[ind:l, ])
   tree$edge <- edge
-  tree$Nnode <- tree$Nnode + 1
+  tree$Nnode <- tree$Nnode + 1L
   tree
 }
 
-
+# raus?
 addOneTree <- function(tree, subtree, i, node) {
   edge <- tree$edge
   parent <- edge[, 1]
@@ -404,23 +398,6 @@ addOneTree <- function(tree, subtree, i, node) {
 }
 
 
-#reorderPruning <- function(x, ...) {
-#  edge <- x$edge
-#  parents <- as.integer(edge[, 1])
-#  child <- as.integer(edge[, 2])
-#  root <- as.integer(parents[!match(parents, child, 0)][1])  # unique out
-#  if (length(root) > 2)
-#    stop("more than 1 root found")
-#  n <- length(parents)
-#  m <- max(edge)  # edge  parents
-#  neworder <- .C("C_reorder", parents, child, as.integer(n), as.integer(m),
-#    integer(n), as.integer(root - 1L), PACKAGE = "phangorn")[[5]]
-#  x$edge <- edge[neworder, ]
-#  x$edge.length <- x$edge.length[neworder]
-#  attr(x, "order") <- "pruningwise"
-#  x
-#}
-
 
 #' Add tips to a tree
 #'
@@ -430,7 +407,7 @@ addOneTree <- function(tree, subtree, i, node) {
 #' @param tree an object of class "phylo".
 #' @param tips a character vector containing the names of the tips.
 #' @param where an integer or character vector of the same length as tips giving
-#' the number of the node or tip of the tree x where the tree y is binded.
+#' the number of the node or tip of the tree where to add the new tips.
 #' @param edge.length optional numeric vector with edge length
 #' @return an object of class phylo
 #' @author Klaus Schliep \email{klaus.schliep@@gmail.com}
@@ -454,7 +431,7 @@ add.tips <- function(tree, tips, where, edge.length = NULL) {
   }
   ind <- match(where, edge[, 2])
 
-  n_internal <- sum(unique(where) <= nTips)
+  n_internal <- as.integer(sum(unique(where) <= nTips))
   edge[edge > nTips] <- edge[edge > nTips] + nTips_new
   p_vec <- integer(max(edge) + n_internal)
   p_vec[edge[, 2]] <- edge[, 1]
@@ -530,8 +507,11 @@ add.tips <- function(tree, tips, where, edge.length = NULL) {
 #' @examples
 #'
 #' trees <- allTrees(5)
+#'
+#' old.par <- par(no.readonly = TRUE)
 #' par(mfrow = c(3,5))
 #' for(i in 1:15)plot(trees[[i]])
+#' par(old.par)
 #'
 #' @export allTrees
 allTrees <- function(n, rooted = FALSE, tip.label = NULL) {
@@ -607,7 +587,7 @@ allTrees <- function(n, rooted = FALSE, tip.label = NULL) {
     edge <- edges[[x]]
     edge <- edge[reorderRcpp(edge, n, n + 1L, 2L), ]
     tree <- list(edge = edge)
-    tree$Nnode <- n - 2L + rooted
+    tree$Nnode <- as.integer(n - 2L + rooted)
     attr(tree, "order") <- "postorder"
     class(tree) <- "phylo"
     trees[[x]] <- tree
@@ -690,7 +670,7 @@ Ancestors <- function(x, node, type = c("all", "parent")) {
   if (type == "parent")
     return(pvector[node])
   anc <- function(pvector, node) {
-    res <- numeric(0)
+    res <- integer(0)
     repeat {
       anc <- pvector[node]
       if (anc == 0) break
