@@ -54,63 +54,12 @@ cophenetic.networx <- function(x) {
 }
 
 
-SHORTwise <- function(x, nTips, delete = FALSE) {
-  v <- 1:nTips
-  l <- lengths(x)
-  lv <- floor(nTips / 2)
-  for (i in seq_along(x)) {
-    if (l[i] > lv) {
-      y <- x[[i]]
-      x[[i]] <- v[-y]
-    }
-    if (l[i] == nTips / 2) {
-      y <- x[[i]]
-      if (y[1] != 1)
-        x[[i]] <- v[-y]
-    }
-  }
-  if (any(l == nTips) && delete) {
-    x <- x[l != nTips]
-  }
-  x
-}
-
-
-oneWise <- function(x, nTips = NULL) {
-  if (is.null(nTips)) nTips <- length(x[[1L]])
-  v <- 1:nTips
-  for (i in seq_along(x)) {
-    y <- x[[i]]
-    if (y[1] != 1)
-      y <- v[-y]
-    if (y[1] != 1)
-      y <- v[-y]
-    x[[i]] <- y
-  }
-  x
-}
-
-
-zeroWise <- function(x, nTips = NULL) {
-  if (is.null(nTips)) nTips <- length(x[[1L]])
-  v <- 1:nTips
-  for (i in seq_along(x)) {
-    y <- x[[i]]
-    if (y[1] == 1)
-      y <- v[-y]
-    x[[i]] <- y
-  }
-  x
-}
-
-
 ## @aliases treedist RF.dist wRF.dist KF.dist path.dist sprdist SPR.dist
-
 #' Distances between trees
 #'
 #' \code{treedist} computes different tree distance methods and \code{RF.dist}
 #' the Robinson-Foulds or symmetric distance. The Robinson-Foulds distance only
-#' depends on the toplogy of the trees. If edge weights should be considered
+#' depends on the topology of the trees. If edge weights should be considered
 #' \code{wRF.dist} calculates the weighted RF distance (Robinson & Foulds
 #' 1981). and \code{KF.dist} calculates the branch score distance (Kuhner &
 #' Felsenstein 1994).  \code{path.dist} computes the path difference metric as
@@ -217,8 +166,8 @@ treedist <- function(tree1, tree2, check.labels = TRUE) {
 
   bp1 <- bip(tree1)
   bp2 <- bip(tree2)
-  bp1 <- SHORTwise(bp1, length(tree1$tip.label))
-  bp2 <- SHORTwise(bp2, length(tree2$tip.label))
+  bp1 <- SHORTwise(bp1)
+  bp2 <- SHORTwise(bp2)
   bp1 <- sapply(bp1, paste, collapse = "_")
   bp2 <- sapply(bp2, paste, collapse = "_")
 
@@ -295,9 +244,9 @@ sprdist <- function(tree1, tree2) {
   if (!is.binary(tree1) | !is.binary(tree2)) message("Trees are not binary!")
   # possibly replace bip with bipart
   bp1 <- bip(tree1)
-  bp1 <- SHORTwise(bp1, lt1)
+  bp1 <- SHORTwise(bp1)
   bp2 <- bip(tree2)
-  bp2 <- SHORTwise(bp2, lt2)
+  bp2 <- SHORTwise(bp2)
 
   bp1 <- bp1[ lengths(bp1) > 1 ] # only internal nodes
   bp2 <- bp2[ lengths(bp2) > 1 ]
@@ -306,6 +255,8 @@ sprdist <- function(tree1, tree2) {
   # OBS: SPR distance works w/ incompatible splits only, but it needs common
   # cherries (to replace by single leaf)
   spr <- .Call("C_sprdist", bp1, bp2, lt1)
+  tmp <- .Call("C_sprdist", bp2, bp1, lt1)[1]
+  spr[1] <- min(spr[1], tmp)
   names(spr) <- c("spr", "spr_extra", "rf", "hdist")
   spr
 }
@@ -320,21 +271,22 @@ SPR1 <- function(trees) {
 
   nTips <- length(trees[[1]]$tip.label)
 
-  fun <- function(x, nTips) {
+  fun <- function(x) {
     bp <- bipart(x)
-    bp <- SHORTwise(bp, nTips)
+    bp <- SHORTwise(bp)
     bp <- bp[ lengths(bp) > 1 ]
     bp
   }
 
-  BP <- lapply(trees, fun, nTips)
+  BP <- lapply(trees, fun)
   k <- 1
   l <- length(trees)
   SPR <- numeric( (l * (l - 1)) / 2)
   for (i in 1:(l - 1)) {
     bp <- BP[[i]]
     for (j in (i + 1):l) {
-      SPR[k] <-  .Call("C_sprdist", bp, BP[[j]], nTips)[1]
+      SPR[k] <-  min( .Call("C_sprdist", bp, BP[[j]], nTips)[1],
+                      .Call("C_sprdist", BP[[j]], bp, nTips)[1])
       k <- k + 1
     }
   }
@@ -360,18 +312,20 @@ SPR2 <- function(tree, trees) {
   if (has.singles(tree)) tree <- collapse.singles(tree)
   nTips <- length(tree$tip.label)
 
-  fun <- function(x, nTips) {
+  fun <- function(x) {
     bp <- bipart(x)
-    bp <- SHORTwise(bp, nTips)
+    bp <- SHORTwise(bp)
     bp <- bp[ lengths(bp) > 1 ]
     bp
   }
 
-  bp <-  fun(tree, nTips)
+  bp <-  fun(tree)
   l <- length(trees)
   SPR <- numeric(l)
   for (i in 1:l) {
-    SPR[i] <- .Call("C_sprdist", bp, fun(trees[[i]], nTips), nTips)[1]
+    bpi <- fun(trees[[i]])
+    SPR[i] <- min(.Call("C_sprdist", bp, bpi, nTips)[1],
+                  .Call("C_sprdist", bpi, bp, nTips)[1])
   }
   if (!is.null(names(trees))) names(SPR) <- names(trees)
   return(SPR)
@@ -414,21 +368,13 @@ wRF0 <- function(tree1, tree2, normalize = FALSE, check.labels = TRUE,
   if (!is.binary(tree1) | !is.binary(tree2))
     message("Trees are not binary!")
   if (check.labels) tree2 <- checkLabels(tree2, tree1$tip.label)
-#  if (check.labels) {
-#    ind <- match(tree1$tip.label, tree2$tip.label)
-#    if (any(is.na(ind)) | length(tree1$tip.label) != length(tree2$tip.label))
-#      stop("trees have different labels")
-#    tree2$tip.label <- tree2$tip.label[ind]
-#    ind2 <- match(seq_along(ind), tree2$edge[, 2])
-#    tree2$edge[ind2, 2] <- order(ind)
-#  }
   if (has.singles(tree1)) tree1 <- collapse.singles(tree1)
   if (has.singles(tree2)) tree2 <- collapse.singles(tree2)
   bp1 <- bip(tree1)
   bp2 <- bip(tree2)
   if (!rooted) {
-    bp1 <- SHORTwise(bp1, length(tree1$tip.label))
-    bp2 <- SHORTwise(bp2, length(tree2$tip.label))
+    bp1 <- SHORTwise(bp1)
+    bp2 <- SHORTwise(bp2)
   }
   bp1 <- sapply(bp1, paste, collapse = "_")
   bp2 <- sapply(bp2, paste, collapse = "_")
@@ -487,7 +433,7 @@ wRF2 <- function(tree, trees, normalize = FALSE, check.labels = TRUE,
 
   fun2 <- function(x, nTips) {
     bp <- bip(x)
-    bp <- SHORTwise(bp, nTips)
+    bp <- SHORTwise(bp)
     bp <- sapply(bp, paste, collapse = "_")
     bp
   }
@@ -503,7 +449,7 @@ wRF2 <- function(tree, trees, normalize = FALSE, check.labels = TRUE,
 
   bp <- bip(tree)
 
-  if (!rooted) bp <- SHORTwise(bp, nTips)
+  if (!rooted) bp <- SHORTwise(bp)
   bp <- sapply(bp, paste, collapse = "_")
 
   w <- numeric(max(tree$edge))
@@ -556,7 +502,7 @@ wRF1 <- function(trees, normalize = FALSE, check.labels = TRUE,
   W <- lapply(trees, fun1)
   fun2 <- function(x, nTips) {
     bp <- bip(x)
-    bp <- SHORTwise(bp, nTips)
+    bp <- SHORTwise(bp)
     bp <- sapply(bp, paste, collapse = "_")
     bp
   }
@@ -598,9 +544,9 @@ wRF1 <- function(trees, normalize = FALSE, check.labels = TRUE,
 mRF2 <- function(tree, trees, normalize = FALSE, check.labels = TRUE,
                  rooted = FALSE) {
   if (!inherits(trees, "multiPhylo"))
-    stop("trees should be an object of class \"multiPhylo\"")
+    stop("Argument trees should be an object of class \"multiPhylo\"")
   if (!inherits(tree, "phylo"))
-    stop("trees should be an object of class \"phylo\"")
+    stop("Argument tree should be an object of class \"phylo\"")
   trees <- .compressTipLabel(trees)
   tipLabel <- attr(trees, "TipLabel")
   if (check.labels) tree <- checkLabels(tree, tipLabel)
@@ -630,10 +576,10 @@ mRF2 <- function(tree, trees, normalize = FALSE, check.labels = TRUE,
   tree <- reorder(tree, "postorder")
   trees <- reorder(trees, "postorder")
   xx <- lapply(trees, bipart)
-  if (!rooted) xx <- lapply(xx, SHORTwise, nTips)
+  if (!rooted) xx <- lapply(xx, SHORTwise)
   xx <- lapply(xx, function(x) sapply(x, paste, collapse = "_"))
   yy <- bipart(tree)
-  if (!rooted) yy <- SHORTwise(yy, nTips)
+  if (!rooted) yy <- SHORTwise(yy)
   yy <- sapply(yy, paste, collapse = "_")
 
   NnodeT <- Nnode(tree)
@@ -656,7 +602,7 @@ mRF2 <- function(tree, trees, normalize = FALSE, check.labels = TRUE,
 
 mRF <- function(trees, normalize = FALSE, rooted = FALSE) {
   if (!inherits(trees, "multiPhylo"))
-    stop("trees should be an object of class \"multiPhylo\"")
+    stop("Argument trees should be an object of class \"multiPhylo\"")
   trees <- .compressTipLabel(trees)
   tipLabel <- attr(trees, "TipLabel")
   nTips <- length(tipLabel)
@@ -664,7 +610,7 @@ mRF <- function(trees, normalize = FALSE, rooted = FALSE) {
   RF <- numeric( (l * (l - 1)) / 2)
 
   if (rooted & any(!is.rooted(trees))) {
-    warning("some trees were rooted, unrooted all")
+    warning("Some trees were rooted, unrooted all")
     rooted <- FALSE
   }
   if (!rooted) {
@@ -691,7 +637,7 @@ mRF <- function(trees, normalize = FALSE, rooted = FALSE) {
   trees <- unclass(trees)
 
   xx <- lapply(trees, bipart)
-  if (!rooted) xx <- lapply(xx, SHORTwise, nTips)
+  if (!rooted) xx <- lapply(xx, SHORTwise)
   xx <- lapply(xx, function(x) sapply(x, paste, collapse = "_"))
   # returns list of character vectors
 
@@ -741,22 +687,13 @@ RF0 <- function(tree1, tree2 = NULL, normalize = FALSE, check.labels = TRUE,
     }
   }
   if (check.labels) tree2 <- checkLabels(tree2, tree1$tip.label)
-#  if (check.labels) {
-#    ind <- match(tree1$tip.label, tree2$tip.label)
-#    if (any(is.na(ind)) | length(tree1$tip.label) !=
-#      length(tree2$tip.label))
-#      stop("trees have different labels")
-#    tree2$tip.label <- tree2$tip.label[ind]
-#    ind2 <- match(seq_along(ind), tree2$edge[, 2])
-#    tree2$edge[ind2, 2] <- order(ind)
-#  }
   if (!is.binary(tree1) | !is.binary(tree2)) message("Trees are not binary!")
   bp1 <- bipart(tree1)
   bp2 <- bipart(tree2)
   nTips <- length(tree1$tip.label)
   if (!rooted) {
-    bp1 <- SHORTwise(bp1, nTips)
-    bp2 <- SHORTwise(bp2, nTips)
+    bp1 <- SHORTwise(bp1)
+    bp2 <- SHORTwise(bp2)
   }
   RF <- sum(match(bp1, bp2, nomatch = 0L) == 0L) +
     sum(match(bp2, bp1, nomatch = 0L) == 0L)
@@ -769,13 +706,13 @@ RF0 <- function(tree1, tree2 = NULL, normalize = FALSE, check.labels = TRUE,
 #' @export
 RF.dist <- function(tree1, tree2 = NULL, normalize = FALSE, check.labels = TRUE,
                     rooted = FALSE) {
-  if (class(tree1) == "phylo" && class(tree2) == "phylo")
+  if (inherits(tree1, "phylo") && inherits(tree2, "phylo"))
     return(RF0(tree1, tree2, normalize, check.labels, rooted))
-  if (class(tree1) == "multiPhylo" && is.null(tree2))
+  if (inherits(tree1, "multiPhylo") && is.null(tree2))
     return(mRF(tree1, normalize, rooted))
-  if (class(tree1) == "phylo" && class(tree2) == "multiPhylo")
+  if (inherits(tree1, "phylo") && inherits(tree2, "multiPhylo"))
     return(mRF2(tree1, tree2, normalize, check.labels, rooted))
-  if (class(tree2) == "phylo" && class(tree1) == "multiPhylo")
+  if (inherits(tree2, "phylo") && inherits(tree1, "multiPhylo"))
     return(mRF2(tree2, tree1, normalize, check.labels, rooted))
   else return(NULL)
 }
@@ -785,13 +722,13 @@ RF.dist <- function(tree1, tree2 = NULL, normalize = FALSE, check.labels = TRUE,
 #' @export
 wRF.dist <- function(tree1, tree2 = NULL, normalize = FALSE,
                      check.labels = TRUE, rooted = FALSE) {
-  if (class(tree1) == "phylo" && class(tree2) == "phylo")
+  if (inherits(tree1, "phylo") && inherits(tree2, "phylo"))
     return(wRF0(tree1, tree2, normalize, check.labels, rooted))
-  if (class(tree1) == "multiPhylo" && is.null(tree2))
+  if (inherits(tree1, "multiPhylo") && is.null(tree2))
     return(wRF1(tree1, normalize, check.labels, rooted))
-  if (class(tree1) == "phylo" && class(tree2) == "multiPhylo")
+  if (inherits(tree1, "phylo") && inherits(tree2, "multiPhylo"))
     return(wRF2(tree1, tree2, normalize, check.labels, rooted))
-  if (class(tree2) == "phylo" && class(tree1) == "multiPhylo")
+  if (inherits(tree2, "phylo") && inherits(tree1, "multiPhylo"))
     return(wRF2(tree2, tree1, normalize, check.labels, rooted))
   else return(NULL)
 }
@@ -820,8 +757,8 @@ kf0 <- function(tree1, tree2, check.labels = TRUE, rooted = FALSE) {
   bp2 <- bip(tree2)
 
   if (!rooted) {
-    bp1 <- SHORTwise(bp1, length(tree1$tip.label))
-    bp2 <- SHORTwise(bp2, length(tree2$tip.label))
+    bp1 <- SHORTwise(bp1)
+    bp2 <- SHORTwise(bp2)
   }
   bp1 <- sapply(bp1, paste, collapse = "_")
   bp2 <- sapply(bp2, paste, collapse = "_")
@@ -875,7 +812,7 @@ kf1 <- function(tree, trees, check.labels = TRUE, rooted = FALSE) {
 
   fun2 <- function(x, nTips) {
     bp <- bip(x)
-    bp <- SHORTwise(bp, nTips)
+    bp <- SHORTwise(bp)
     bp <- sapply(bp, paste, collapse = "_")
     bp
   }
@@ -889,7 +826,7 @@ kf1 <- function(tree, trees, check.labels = TRUE, rooted = FALSE) {
 
   if (!rooted & is.rooted(tree)) tree <- unroot(tree)
   bp <- bip(tree)
-  if (!rooted) bp <- SHORTwise(bp, nTips)
+  if (!rooted) bp <- SHORTwise(bp)
   bp <- sapply(bp, paste, collapse = "_")
 
   w <- numeric(max(tree$edge))
@@ -937,7 +874,7 @@ kf2 <- function(trees, check.labels = TRUE, rooted = FALSE) {
 
   fun2 <- function(x, nTips) {
     bp <- bip(x)
-    bp <- SHORTwise(bp, nTips)
+    bp <- SHORTwise(bp)
     bp <- sapply(bp, paste, collapse = "_")
     bp
   }
